package com.incubator.game.room;

import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.core.room.GRoomInterface;
import com.incubator.game.data.po.RoomPO;
import com.incubator.game.player.Player;
import com.incubator.game.util.GDUtils;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 房间实体基类
 */
public class Room implements GRoomInterface {

    /** 房间内数据 **/
    public RoomPO data;
    /** 房间角色 [key:玩家id, value:玩家对象]**/
    public Map<String, Player> actors = new ConcurrentHashMap<>();

    /** 房间规则数据 **/
    public JSONObject jsonData;

    /** 激活标记 **/
    private boolean isActive;
    /** 检测标记 **/
    public boolean flag;
    /** 时间秒数 **/
    public int time;
    /** 房间状态(0等待 1开局 2进行中 3已结束) **/
    public int state;
    /** 并发锁 **/
    public final Lock lock = new ReentrantLock();
    /** 换位缓存 [key:同意的玩家, value:发起的玩家]**/
    public Map<String, String> changePosMap = new ConcurrentHashMap<>();

    /** 回放数据 **/
    public JSONObject playBack = new JSONObject();

    /** 房间任务主线程 **/
    private ScheduledExecutorService scheduler;

    public Room() {
        this.scheduler = Executors.newSingleThreadScheduledExecutor();
    }

    /** 初始化 **/
    public void init(Player player, JSONObject jsonData) {}

    /** 加入房间 **/
    public void joinRoom(Player player) {}

    /** 离开房间 **/
    public void leaveRoom(Player player) {}

    /**
     * 开启房间主线程任务
     */
    public void startRoomTask() {
        this.isActive = true;
        if (scheduler == null) {
            this.scheduler = Executors.newSingleThreadScheduledExecutor();
        }
        this.scheduler.scheduleAtFixedRate(() -> {
            if (this.isActive) {
                this.doUpdate();
            }
        }, 0, 1, TimeUnit.SECONDS);
    }

    public void initCardList() {}

    /**
     * 房间主线程逻辑
     */
    public void doUpdate() {}

    public void doSendCard() {}

    public void doStart() {}

    public void playerDisCard(Player player, int[] disCardList, GDUtils.CardType disCardType) {}

    public void playerPass(Player player, boolean isAuto) {}

    public boolean checkBackTributeStart() {
        return false;
    }

    /** 玩家换座位 **/
    public synchronized void switchSeat(Player player, int targetSeat) {
        lock.lock();
        try {
            if (targetSeat < 0 || targetSeat >= this.data.maxNum) {
                return; // 目标座位无效
            }
            // 找到玩家当前座位
            int currentSeat = -1;
            for (int i = 0; i < this.data.maxNum; i++) {
                Player tmPlayer = this.data.playerMap[i];
                if (tmPlayer != null && tmPlayer.getId().equals(player.getId())) {
                    currentSeat = i;
                    break;
                }
            }
            if (currentSeat == -1) {
                return; // 玩家不在座位上
            }

            // 检查目标座位
            Player targetPlayer = this.data.playerMap[targetSeat];
            if (targetPlayer == null) {
                player.data.pos = targetSeat;
                // 如果目标座位为空，直接换过去
                this.data.playerMap[targetSeat] = player;
                this.data.playerMap[currentSeat] = null;
                Log.debug("roomId: " + this.data.roomId + ", " + player.data.name + " 换到座位号: " + targetSeat);
            }
            // 如果目标座位已被占用，与目标玩家交换位置
            if (targetPlayer != null && targetPlayer.data.isChangePos) {
                player.data.pos = targetSeat;
                targetPlayer.data.pos = currentSeat;
                this.data.playerMap[targetSeat] = player;
                this.data.playerMap[currentSeat] = targetPlayer;
                Log.debug("roomId: " + this.data.roomId + ", " + player.data.name + " 与 " + targetPlayer.data.name + " 交换了座位");
            }
        } finally {
            lock.unlock();
        }
    }

    /**
     * 房间重置逻辑
     */
    void resetRoom() {}

    /**
     * 重置房间，用于对象池回收前清理状态
     */
    public void reset() {
        this.isActive = false;
    }

    /**
     * 销毁房间，释放资源
     */
    public void destroy() {
        this.reset();
        this.scheduler.shutdownNow();
        this.scheduler = null;
        // 从缓存移除
        RoomService.getInstance().roomMap.remove(this.data.roomId);
        // 重置房间数据
        this.resetRoom();
        // 归还房间对象
        RoomService.getInstance().roomPool.releaseRoom(this);
        Log.debug("Room {} has been destroyed.", this.data.roomId);
    }

    public Map<String, Object> roomToMessage(Player player, Player opPlayer) {
        return null;
    }

    public Map<String, Object> playerRoomToMessage(Player player, boolean flag) {
        return null;
    }
}
